<?php
global $wp;
$plugin = $GLOBALS['this_plugin'];
$banner = $GLOBALS['banner'];
$bannerMeta = isset($banner['meta']) ? $banner['meta'] : array();
$config = unserialize( get_option( $plugin->the_plugin->alias . '_settings' ) );

$template_path = $plugin->the_plugin->cfg['paths']['design_dir_url'] .'/template_'. $bannerMeta['theme'] . '/';

if( isset($_POST['validate']) ) {
	require( $plugin->the_plugin->cfg['paths']['frontend_dir_path'] . '/ajax_validation.php' );
	$validate = $ageValidation->validateAgeRestriction();
	  
	if( $validate === true ) {   
		wp_safe_redirect( home_url($wp->request) );
	}else if( $_REQUEST['age_restriction_confirmation'] == 'no' && (isset($bannerMeta['redirect_under_age']) && trim($bannerMeta['redirect_under_age']) != '') ) {
		$redirect_under_age = esc_url( $bannerMeta['redirect_under_age'] );
		wp_redirect( $redirect_under_age );
	}
}

// Background Image
if ( isset($bannerMeta['background_image']) && !empty($bannerMeta['background_image']) ) {
	if( (int) $bannerMeta['background_image'] == 0 && trim($bannerMeta['background_image']) != "" ){
		$image = array( $bannerMeta['background_image'] ); 
	}else{
		$image = wp_get_attachment_image_src( (int) $bannerMeta['background_image'], 'full' );
	}
	if ( isset($image[0]) && !empty($image[0]) ) $background_image = $image[0];  
}

// Convert hexdec color string to rgb(a) string */
function agerestriction_hex2rgba($color, $opacity = false) {
 
	$default = 'rgb(0,0,0)';
 
	//Return default if no color provided
	if(empty($color))
          return $default; 
 
	//Sanitize $color if "#" is provided 
        if ($color[0] == '#' ) {
        	$color = substr( $color, 1 );
        }
 
        //Check if color has 6 or 3 characters and get values
        if (strlen($color) == 6) {
                $hex = array( $color[0] . $color[1], $color[2] . $color[3], $color[4] . $color[5] );
        } elseif ( strlen( $color ) == 3 ) {
                $hex = array( $color[0] . $color[0], $color[1] . $color[1], $color[2] . $color[2] );
        } else {
                return $default;
        }
 
        //Convert hexadec to rgb
        $rgb =  array_map('hexdec', $hex);
 
        //Check if opacity is set(rgba or rgb)
        if($opacity){
        	if(abs($opacity) > 1)
        		$opacity = 1.0;
        	$output = 'rgba('.implode(",",$rgb).','.$opacity.')';
        } else {
        	$output = 'rgb('.implode(",",$rgb).')';
        }
 
        //Return rgb(a) color string
        return $output;
}

// Logo
if ( isset($bannerMeta['logo']) && !empty($bannerMeta['logo']) ) {
	
	if( (int) $bannerMeta['logo'] == 0 && trim($bannerMeta['logo']) != "" ){
		$image = array( $bannerMeta['logo'] ); 
	}else{
		$image = wp_get_attachment_image_src( (int) $bannerMeta['logo'], 'full' );
	}
	if ( isset($image[0]) && !empty($image[0]) ) $banner_logo = $image[0];  
}

// Replace shortcodes in before & after text
if( isset($bannerMeta['text_before']) && strstr($bannerMeta['text_before'], '[minimum_age]') ) {
	$bannerMeta['text_before'] = str_replace('[minimum_age]', $bannerMeta['minimum_age'], $bannerMeta['text_before']);
}
if( isset($bannerMeta['text_after']) && strstr($bannerMeta['text_after'], '[minimum_age]') ) {
	$bannerMeta['text_after'] = str_replace('[minimum_age]', $bannerMeta['minimum_age'], $bannerMeta['text_after']);
}

add_action( 'wp_print_styles', 'print_template_styles', 100 );
add_action( 'wp_print_scripts', 'print_template_scripts', 100 );

function print_template_styles() {
	$banner = $GLOBALS['banner'];
	$bannerMeta = isset($banner['meta']) ? $banner['meta'] : array();
	$template_path = $GLOBALS['this_plugin']->the_plugin->cfg['paths']['design_dir_url'] .'/template_'. $bannerMeta['theme'] . '/';
	
	if( isset($bannerMeta['typography']['font_family']) &&  trim($bannerMeta['typography']['font_family']) != '' ) {
		wp_enqueue_style('google-fonts', '//fonts.googleapis.com/css?family=' . ( urlencode($bannerMeta["typography"]["font_family"]) ) .':400,600,700,800');
	}else{
		wp_enqueue_style('google-fonts', '//fonts.googleapis.com/css?family=Open+Sans:400,600,700,800');
	}
	
	wp_enqueue_style('reset', $template_path . 'css/reset.css');
	wp_enqueue_style('main', $template_path . 'css/style.css');
	wp_enqueue_style('responsive', $template_path . 'css/responsive.css');
		
}


function print_template_scripts() {
	$plugin = $GLOBALS['this_plugin'];
	$banner = $GLOBALS['banner'];
	$bannerMeta = isset($banner['meta']) ? $banner['meta'] : array();
	$template_path = $GLOBALS['this_plugin']->the_plugin->cfg['paths']['design_dir_url'] .'/template_'. $bannerMeta['theme'] . '/';
	$config = unserialize( get_option( $plugin->the_plugin->alias . '_settings' ) );
	  
	wp_enqueue_script('jquery');
	wp_enqueue_script('mobile.detect', $template_path . 'js/mobile.detect.js', array('jquery'));
	wp_enqueue_script('app-class', $template_path . '/js/app.class.js', array('jquery'));
	
	if( (isset($bannerMeta['connect_w_google']) && $bannerMeta['connect_w_google'] == 'yes') && 
		(isset($config['google_client_id']) && trim($config['google_client_id']) != '') 
	) {
		wp_enqueue_script('google-apis-client', 'https://apis.google.com/js/client:platform.js', array('jquery'));
	}	
}
?>

<!DOCTYPE html>
<html>
<head>
	<meta charset="utf-8">
	<meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1">
  
	<title><?php echo isset($bannerMeta['page_title']) ? esc_attr($bannerMeta['page_title']) : esc_html__('Age restriction', 'aat-age-restriction'); ?></title>
	
	<?php echo isset($bannerMeta['meta_description']) && trim($bannerMeta['meta_description']) != '' ? '<meta name="description" content="' . esc_attr($bannerMeta['meta_description']) . '">' : ''; ?>
	<?php echo isset($bannerMeta['meta_keywords']) && trim($bannerMeta['meta_keywords']) != '' ? '<meta name="keywords" content="' . esc_attr($bannerMeta['meta_keywords']) . '">' : ''; ?>
	
	<?php wp_head(); ?>
	
	<script type="text/javascript">
		var gdpr_error_msg = '<?php echo isset($bannerMeta['gdpr_error_consent_text']) && trim($bannerMeta['gdpr_error_consent_text']) != '' ? do_shortcode($bannerMeta['gdpr_error_consent_text']) : esc_html__('You must agree to GDPR terms in order to continue.', 'aat-age-restriction'); ?>';
	</script>
	
	
	<style type="text/css">
		<?php if( isset($bannerMeta['typography']['font_family']) &&  trim($bannerMeta['typography']['font_family']) != '' ) { ?>
		.ar-box, .styledSelect, .options {
			font-family: '<?php echo esc_attr($bannerMeta['typography']['font_family']); ?>';
		}
		<?php } ?>
		
		<?php if( isset($background_image) || isset($bannerMeta['background_color']) ) { ?>
		#wrapper {
			<?php if( isset($background_image) ) { ?>
			background-image: url(<?php echo esc_url($background_image); ?>);
			<?php }else if( isset($bannerMeta['background_color']) ) { ?>
			background: #<?php echo esc_attr($bannerMeta['background_color']); ?>;
			<?php } ?>
		}
		<?php } ?>
		
		<?php if( isset($bannerMeta['box_background_color']) &&  trim($bannerMeta['box_background_color']) != '' ) { ?>
		.ar-box {
			background-color: <?php 
									if( isset($bannerMeta['box_background_opacity']) &&  trim($bannerMeta['box_background_opacity']) != '' ) {
										echo agerestriction_hex2rgba( $bannerMeta['box_background_color'], $bannerMeta['box_background_opacity'] / 100 );
									}
									else {
										echo agerestriction_hex2rgba($bannerMeta['box_background_color']);
									}
								?>
		}
		<?php } ?>
		
		<?php if( isset($bannerMeta['title_text_color']) && trim($bannerMeta['title_text_color']) != '' ) { ?>
		.ar-box h1 {
			color: #<?php echo esc_attr($bannerMeta['title_text_color']); ?>;
		}
		<?php } ?>
		
		<?php if( isset($bannerMeta['text_color_before']) && trim($bannerMeta['text_color_before']) != '' ) { ?>
		.ar-box .text-before {
			color: #<?php echo esc_attr($bannerMeta['text_color_before']); ?>;
		}
		<?php } ?>
		
		<?php if( isset($bannerMeta['text_color_after']) && trim($bannerMeta['text_color_after']) != '' ) { ?>
		.ar-box .text-after {
			color: #<?php echo esc_attr($bannerMeta['text_color_after']); ?>;
		}
		<?php } ?>
		
		<?php if( isset($bannerMeta['text_color']) && trim($bannerMeta['text_color']) != '' ) { ?>
		.ar-box .ar-form-row input[type="text"], .ar-box .ar-remember-me, .ar-box .ar-form-row select {
			color: #<?php echo esc_attr($bannerMeta['text_color']); ?>;
		}
		.ar-box .ar-form-row input[type="text"]::-webkit-input-placeholder { /* WebKit browsers */
		    color:    #<?php echo esc_attr($bannerMeta['text_color']); ?>;
		}
		.ar-box .ar-form-row input[type="text"]:-moz-placeholder { /* Mozilla Firefox 4 to 18 */
		   color:    #<?php echo esc_attr($bannerMeta['text_color']); ?>;
		   opacity:  1;
		}
		.ar-box .ar-form-row input[type="text"]::-moz-placeholder { /* Mozilla Firefox 19+ */
		   color:    #<?php echo esc_attr($bannerMeta['text_color']); ?>;
		   opacity:  1;
		}
		.ar-box .ar-form-row input[type="text"]:-ms-input-placeholder { /* Internet Explorer 10+ */
		   color:    #<?php echo esc_attr($bannerMeta['text_color']); ?>;
		}
		<?php } ?>
		
		<?php if( isset($bannerMeta['text_hover_color']) && trim($bannerMeta['text_hover_color']) != '' ) { ?>
		.ar-box .ar-form-row label:hover, .ar-box .ar-form-row label.active {
			color: #<?php echo esc_attr($bannerMeta['text_hover_color']); ?>;
		}
		<?php } ?>
		
		<?php if( (isset($bannerMeta['enter_btn_bg_color']) && trim($bannerMeta['enter_btn_bg_color']) != '') ||
				  (isset($bannerMeta['enter_btn_text_color']) && trim($bannerMeta['enter_btn_text_color']) != '') 
		) { ?>
		.ar-box .ar-submit {
			<?php if( isset($bannerMeta['enter_btn_bg_color']) && trim($bannerMeta['enter_btn_bg_color']) != '' ) { ?>
			background-color: #<?php echo esc_attr($bannerMeta['enter_btn_bg_color']); ?>;
			<?php } ?>
			
			<?php if( isset($bannerMeta['enter_btn_text_color']) && trim($bannerMeta['enter_btn_text_color']) != '' ) { ?>
			color: #<?php echo esc_attr($bannerMeta['enter_btn_text_color']); ?>;
			<?php } ?>
		}
		<?php } ?>
		
		<?php
		if( isset($bannerMeta['custom_css']) && trim($bannerMeta['custom_css']) != '' ) {
			echo $bannerMeta['custom_css'];
		}
		?>
	</style>
</head>
<body>
	<?php 
	if( (isset($bannerMeta['connect_w_facebook']) && $bannerMeta['connect_w_facebook'] == 'yes') && 
		(isset($config['fb_app_id']) && trim($config['fb_app_id']) != '') 
	) {
	?>
	<script type="text/javascript">
		window.fbAsyncInit = function() {
			FB.init({
				appId      : '<?php echo esc_attr($config['fb_app_id']); ?>',
				autoLogAppEvents : true,
				xfbml      : true,
				version    : 'v2.10'
			});
		};
		
		(function(d, s, id){
			var js, fjs = d.getElementsByTagName(s)[0];
			if (d.getElementById(id)) {return;}
			js = d.createElement(s); js.id = id;
			js.src = "//connect.facebook.net/en_US/sdk.js";
			fjs.parentNode.insertBefore(js, fjs);
		}(document, 'script', 'facebook-jssdk'));
	</script>
	<?php } ?>
	
	<div class="preloader"><p><?php esc_html_e('Loading...', 'aat-age-restriction'); ?></p></div>
	
	<div id="wrapper">
		<div class="ar-box">
			<?php if( isset($banner_logo) ) { ?>
			<div class="ar-logo">
				<div class="logo-wrapper"><img src="<?php echo esc_url($banner_logo); ?>"/></div>
			</div>
			<?php } ?>
			
			<?php echo isset($bannerMeta['text_before']) && !empty($bannerMeta['text_before']) ? '<h2 class="text-before">' . do_shortcode($bannerMeta['text_before']) . '</h2>' : ''; ?>
			
			<h1><?php echo isset($bannerMeta['restriction_title']) && !empty($bannerMeta['restriction_title']) ? do_shortcode($bannerMeta['restriction_title']) : ''; ?></h1>
			
			<?php
          	if( isset($validate) && $validate !== true ) {
          		echo '<span class="errors">&raquo; ' . ( do_shortcode($validate) ) . ' &laquo;</span>';
          	}
          	?>
          	
			<form method="post" action="<?php home_url($wp->request); ?>" id="agecheck">
				<input type="hidden" name="validate" value="<?php echo esc_attr($banner['ID']); ?>" />
				<input type="hidden" id="min-age" value="<?php echo esc_attr($bannerMeta['minimum_age']);?>" />
				<input type="hidden" name="social_details" id="social_details" value="" />
				<input type="hidden" name="age_restriction_day" value=""/>
				<input type="hidden" name="age_restriction_month" value=""/>
				<input type="hidden" name="age_restriction_year" value=""/>
				
				<?php if( $bannerMeta['country_selection'] == 'yes' ) { ?>
				<div class="ar-form-row">
					<select name="age_restriction_country">
                    	<option value=""><?php esc_html_e('Select Country', 'aat-age-restriction'); ?></option>
                    	<?php
						foreach ( $plugin->the_plugin->getCountriesList('code') as $key => $val ) {
							if( isset($bannerMeta['selected_countries']) && is_array($bannerMeta['selected_countries']) && 
								in_array($key, $bannerMeta['selected_countries']) 
							) {
								echo '<option value="'. $key .'|' .$val.'" '. ( isset($_REQUEST['age_restriction_country']) && esc_attr($_REQUEST['age_restriction_country']) == $key.'|'.$val ? 'selected="selected"' : '' ) .'>'. $val .'</option>';
							}else if( !isset($bannerMeta['selected_countries']) ){
								echo '<option value="'. $key .'|'. $val.'" '. ( isset($_REQUEST['age_restriction_country']) && esc_attr($_REQUEST['age_restriction_country']) == $key.'|'.$val ? 'selected="selected"' : '' ) .'>'. $val .'</option>';
							}
						}
                    	?>
					</select>
				</div>
				<br/>
				<?php } ?>
				
				<?php
				if( isset($bannerMeta['gdpr_consent']) && $bannerMeta['gdpr_consent'] == 'yes' ) {
					$gdpr_consent[] = "<div class=\"gdpr_consent\">";
					$gdpr_consent[] = 	isset($bannerMeta['gdpr_consent_text']) && trim($bannerMeta['gdpr_consent_text']) != '' ? '<p>' . do_shortcode($bannerMeta['gdpr_consent_text']) . '</p>' : '';
					$gdpr_consent[] = 	"<input type=\"checkbox\" name=\"gdpr_consent\" id=\"ar-gdpr-consent\" class=\"css-checkbox\">"; 
					$gdpr_consent[] = 	"<label for=\"ar-gdpr-consent\" name=\"gdpr_consent\" class=\"ar-remember-me css-label lite-green-check\">";
					$gdpr_consent[] = 		isset($bannerMeta['gdpr_agree_checkbox_label']) && trim($bannerMeta['gdpr_agree_checkbox_label']) != '' ? do_shortcode(trim($bannerMeta['gdpr_agree_checkbox_label'])) : esc_html__('I agree', 'aat-age-restriction');
					$gdpr_consent[] = 	"</label>";
					$gdpr_consent[] = "</div>";
					
					echo implode("\n", $gdpr_consent);
				}
				?>
				
				<div class="ar-form-row">
					<input type="hidden" name="age_restriction_confirmation" value=""/>
					<a href="#" data-value="yes" class="agecheck_submit"><?php esc_html_e('Yes', 'aat-age-restriction'); ?></a>
					<a href="#" data-value="no" class="agecheck_submit underage"><?php esc_html_e('No', 'aat-age-restriction'); ?></a>
				</div>
				
				<?php
				//check if "Enable Remember me" button is enabled
				if( $bannerMeta['enable_remember_me'] == 'yes' ){
					
					$rememberMe[] = "<div class=\"rememberme\">";
					$rememberMe[] = 		"<input type=\"checkbox\" name=\"remember_me\" id=\"ar-remember-me\" class=\"css-checkbox\" value=\"on\">"; 
					$rememberMe[] = 		"<label for=\"ar-remember-me\" name=\"rememberme\" class=\"ar-remember-me css-label lite-green-check\">";
					$rememberMe[] = 				esc_html__('Remember me?', 'aat-age-restriction');
					$rememberMe[] = 		"</label>";
					$rememberMe[] = "</div>";
					
					echo implode("\n", $rememberMe);
				}
				?>		
			</form>
			
			<?php echo isset($bannerMeta['text_after']) && !empty($bannerMeta['text_after']) ? '<h3 class="text-after">' . do_shortcode($bannerMeta['text_after']) . '</h3>' : ''; ?>
			
			<?php
	    	if( (isset($bannerMeta['connect_w_facebook']) && $bannerMeta['connect_w_facebook'] == 'yes') && 
				(isset($config['fb_app_id']) && trim($config['fb_app_id']) != '') 
			) {
	    	?>
	    		<a href="#" id="age_restriction_fbconnect" class="ar-facebook"><?php echo isset($bannerMeta['connect_w_facebook_btnText']) && trim($bannerMeta['connect_w_facebook_btnText']) != '' ? do_shortcode($bannerMeta['connect_w_facebook_btnText']) : esc_html__('Connect with Facebook', 'aat-age-restriction'); ?></a><br/>
	    	<?php } ?>
	    	
	    	<?php
	    	if( (isset($bannerMeta['connect_w_google']) && $bannerMeta['connect_w_google'] == 'yes') && 
				(isset($config['google_client_id']) && trim($config['google_client_id']) != '') 
			) {
	    	?>
	    		<a href="#" id="age_restriction_gplusconnect" class="ar-google"><?php echo isset($bannerMeta['connect_w_google_btnText']) && trim($bannerMeta['connect_w_google_btnText']) != '' ? do_shortcode($bannerMeta['connect_w_google_btnText']) : esc_html__('Connect with Google+', 'aat-age-restriction'); ?></a>
	    	<?php } ?>
	    </div>
	</div>
	
	<?php
	if( (isset($bannerMeta['connect_w_google']) && $bannerMeta['connect_w_google'] == 'yes') && 
		(isset($config['google_client_id']) && trim($config['google_client_id']) != '') 
	) {
	?>
		
		<input type="hidden" id="google_client_id" name="google_client_id" value="<?php echo esc_attr($config['google_client_id']); ?>"/>
		
	<?php } ?>
    <div id="fade"></div>
    <div class="popup-message">
    	<div class="popup_content"></div>
    	<a href="#" class="btn close"><?php esc_html_e('OK', 'aat-age-restriction'); ?></a>
    </div>
</body>
</html>