<?php 
/**
 * age_restrictionBannersType class
 * ================
 *
 * @author		Andrei Dinca, AA-Team
 * @version		1.0
 * @access 		public
 * @return 		void
 */  
!defined('ABSPATH') and exit;
if (class_exists('age_restrictionBannersType') != true) {
    class age_restrictionBannersType
    {
        /*
        * Some required plugin information
        */
        const VERSION = '1.0';

        /*
        * Store some helpers config
        */
        public $the_plugin = null;
		public $amzHelper = null;

		private $module_folder = '';
		private $module_folder_path = '';

		static protected $_instance;
		
		public $postid = 0;
		private $elements = array();
		private $tabs = array();

        /*
        * Required __construct() function that initalizes the AA-Team Framework
        */
        public function __construct( $the_plugin, $postid=0 )
        {
			$this->the_plugin = $the_plugin;
			if ( isset($this->the_plugin->amzHelper) && !empty($this->the_plugin->amzHelper) ) {
				$this->amzHelper = $this->the_plugin->amzHelper;
			} else {
				require_once( $this->the_plugin->cfg['paths']['plugin_dir_path'] . 'aa-framework/amz.helper.class.php' );
				if( class_exists('age_restrictionAmazonHelper') ){
					$this->amzHelper = age_restrictionAmazonHelper::getInstance( $this->the_plugin, false);
				}
			}
			
			$this->module_folder = $this->the_plugin->cfg['paths']['plugin_dir_url'] . 'modules/restrictions_manager/';
			$this->module_folder_path = $this->the_plugin->cfg['paths']['plugin_dir_path'] . 'modules/restrictions_manager/';

			$this->set_banner_postid( $postid );
			
			$this->config = unserialize( get_option( $this->the_plugin->alias . '_settings' ) );
			  
			if ( $this->the_plugin->is_admin === true ) {
			}
        }
		
		/**
	    * Singleton pattern
	    *
	    * @return age_restrictionBannersType Singleton instance
	    */
	    static public function getInstance()
	    {
	        if (!self::$_instance) {
	            self::$_instance = new self;
	        }

	        return self::$_instance;
	    }
		
		
		/**
		 * Options
		 */
		private function setup() {
			$current_utc_datetime = gmdate("M d Y H:i:s");

			// elements
			$this->elements = array(
				/* commons - design */
				'theme' => array(
					'type' 		=> 'image_picker',
					'std' 		=> '1',
					'title' 	=> esc_html__('Theme:', 'aat-age-restriction'),
					'desc' 		=> esc_html__('Select theme to use.', 'aat-age-restriction'),
					'options'	=> array( 1 => 'Beer', 2 => 'Winery', 3 => 'Adult', 4 => 'Smoking', 'gambling' => 'Gambling', 'r_rated' => 'R - Rated', 'weed' => 'Weed' ),
					'size' => 'large',
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom')
				),
				
				'logo' => array(
					'type' 		=> 'upload_image_wp',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Logo:', 'aat-age-restriction'),
					'value' 	=> esc_html__('Upload image', 'aat-age-restriction'),
					'thumbSize' => array(
						'w' => '100',
						'h' => '100',
						'zc' => '2',
					),
					'desc' 		=> esc_html__('Banner logo.', 'aat-age-restriction'),
					'preview_size'	=> 'thumbnail',
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				'background_image' => array(
					'type' 		=> 'upload_image_wp',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Background image:', 'aat-age-restriction'),
					'value' 	=> esc_html__('Upload image', 'aat-age-restriction'),
					'thumbSize' => array(
						'w' => '100',
						'h' => '100',
						'zc' => '2',
					),
					'desc' 		=> esc_html__('If not set, a default one will be used according to each theme style.', 'aat-age-restriction'),
					'preview_size'	=> 'thumbnail',
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				'background_color' => array(
					'type' 		=> 'color_picker',
					'std'		=> '000000',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Background color:', 'aat-age-restriction'),
					'desc' 		=> esc_html__('If background color is set, the background image will not be used.', 'aat-age-restriction'),
					'cssclass'	=> array('popup')
				),
				
				'box_background_color' => array(
					'type' 		=> 'color_picker',
					'std'		=> 'FFFFFF',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Box Background color:', 'aat-age-restriction'),
					'desc' 		=> esc_html__('If no background color is set for the box, the background will be transparent.', 'aat-age-restriction'),
					'cssclass'	=> array('popup')
				),

				'box_background_opacity' => array(
					'type' 		=> 'range_input',
					'std'		=> '100',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Box Background opacity:', 'aat-age-restriction'),
					'desc' 		=> esc_html__('If no background opacity is set for the box, the default opacity will be 1.', 'aat-age-restriction'),
					'cssclass'	=> array('popup')
				),
				
				'text_color' => array(
					'type' 		=> 'color_picker',
					'std'		=> '363636',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Text color:', 'aat-age-restriction'),
					'desc' 		=> '&nbsp;',
					'cssclass'	=> array('popup')
				),
				
				'text_hover_color' => array(
					'type' 		=> 'color_picker',
					'std'		=> '3c3839',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Hover/Active text color:', 'aat-age-restriction'),
					'desc' 		=> '&nbsp;',
					'cssclass'	=> array('popup')
				),
				
				'country_selection' => array(
					'type' 		=> 'select',
					'std' 		=> 'yes',
					'size' 		=> 'large',
					'force_width'=> '100',
					'title' 	=> esc_html__('Show Country Selection:', 'aat-age-restriction'),
					'desc' 		=> esc_html__('Enable Country Selection', 'aat-age-restriction'),
					'options'	=> array(
						'yes' 			=> esc_html__('YES', 'aat-age-restriction'),
						'no' 			=> esc_html__('NO', 'aat-age-restriction'),
					),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				'date_format' => array(
					'type' 		=> 'select',
					'std' 		=> 'MM/DD/YYYY',
					'size' 		=> 'large',
					'force_width'=> '130',
					'title' 	=> esc_html__('Date format:', 'aat-age-restriction'),
					'desc' 		=> esc_html__('Order the fields', 'aat-age-restriction'),
					'options'	=> array(
						'MM/DD/YYYY' 	=> 'MM/DD/YYYY',
						'DD/MM/YYYY' 	=> 'DD/MM/YYYY',
						'YYYY/DD/MM' 	=> 'YYYY/DD/MM',
						'YYYY/MM/DD' 	=> 'YYYY/MM/DD',
					),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				'gdpr_consent' => array(
					'type' 		=> 'select',
					'std' 		=> 'yes',
					'size' 		=> 'large',
					'force_width'=> '100',
					'title' 	=> esc_html__('Enable GDPR consent', 'aat-age-restriction'),
					'options'	=> array(
						'yes' 			=> esc_html__('YES', 'aat-age-restriction'),
						'no' 			=> esc_html__('NO', 'aat-age-restriction'),
					),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				'gdpr_consent_text' => array(
					'type' 		=> 'textarea-wysiwyg',
					'size' 		=> 'large',
					'title' 	=> esc_html__('GDPR consent text:', 'aat-age-restriction'),
					'std'		=> esc_html__('This form requires your consent in order to continue. Do you agree with our website storing some personal data about you? You can always request us to delete any personal data you might have about you.', 'aat-age-restriction'),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				'gdpr_error_consent_text' => array(
					'type' 		=> 'text',
					'size' 		=> 'large',
					'title' 	=> esc_html__('GDPR error consent text:', 'aat-age-restriction'),
					'std'		=> esc_html__('You must agree to GDPR terms in order to continue.', 'aat-age-restriction'),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				'gdpr_agree_checkbox_label' => array(
					'type' 		=> 'text',
					'size' 		=> 'small',
					'title' 	=> esc_html__('GDPR agree checkbox label:', 'aat-age-restriction'),
					'std'		=> esc_html__('I agree', 'aat-age-restriction'),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				'typography' => array(
					'type' 		=> 'typo',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Typograhpy:', 'aat-age-restriction'),
					'std'		=> 'Open Sans',
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				'restriction_title' => array(
					'type' 		=> 'text',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Title:', 'aat-age-restriction'),
					'std'		=> esc_html__('Please select your date of birth', 'aat-age-restriction'),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				'title_text_color' => array(
					'type' 		=> 'color_picker',
					'std'		=> '363636',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Title text color:', 'aat-age-restriction'),
					'desc' 		=> '&nbsp;',
					'cssclass'	=> array('popup')
				),
				
				'text_before' => array(
					'type' 		=> 'textarea-wysiwyg',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Text before:', 'aat-age-restriction'),
					'std'		=> esc_html__('In order to view this page you must be at least [minimum_age] years old.', 'aat-age-restriction'),
					'desc' 		=> esc_html__('Custom text before birth date selection.', 'aat-age-restriction'),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				'text_color_before' => array(
					'type' 		=> 'color_picker',
					'std'		=> '363636',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Text color before:', 'aat-age-restriction'),
					'desc' 		=> '&nbsp;',
					'cssclass'	=> array('popup')
				),
				
				'text_after' => array(
					'type' 		=> 'textarea-wysiwyg',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Text after:', 'aat-age-restriction'),
					'std'		=> '',
					'desc' 		=> esc_html__('Custom text after "ENTER" button.', 'aat-age-restriction'),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				'text_color_after' => array(
					'type' 		=> 'color_picker',
					'std'		=> '363636',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Text color after:', 'aat-age-restriction'),
					'desc' 		=> '&nbsp;',
					'cssclass'	=> array('popup')
				),
				
				'enter_btn_title' => array(
					'type' 		=> 'text',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Submit button title:', 'aat-age-restriction'),
					'std'		=> 'Verify',
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				'enter_btn_bg_color' => array(
					'type' 		=> 'color_picker',
					'std'		=> '3b393a',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Submit button background color:', 'aat-age-restriction'),
					'desc' 		=> '&nbsp;',
					'cssclass'	=> array('popup')
				),
				
				'enter_btn_text_color' => array(
					'type' 		=> 'color_picker',
					'std'		=> 'FFFFFF',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Submit button text color:', 'aat-age-restriction'),
					'desc' 		=> '&nbsp;',
					'cssclass'	=> array('popup')
				),
				'minimum_age_error_message' => array(
					'type' 		=> 'text',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Minimum age ERROR message:', 'aat-age-restriction'),
					'std'		=> esc_html__('Minimum age required', 'aat-age-restriction'),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				/* commons - custom css */
				'custom_css' => array(
					'type' 		=> 'textarea',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Custom CSS:', 'aat-age-restriction'),
					'std'		=> '',
					'desc' 		=> esc_html__('Restriction Custom CSS.', 'aat-age-restriction'),
					'height'	=> '400px',
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				/* commons - limitations */
				'minimum_age' => array(
					'type' 		=> 'text',
					'size' 		=> 'large',
					'force_width' => 50,
					'title' 	=> esc_html__('Minimum Age:', 'aat-age-restriction'),
					'desc' 		=> esc_html__('This is usefull if you want to restrict to a specific age on a specific country(s).', 'aat-age-restriction'),
					'std'		=> '18',
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				'redirect_under_age' => array(
					'type' 		=> 'text',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Redirect if under age:', 'aat-age-restriction'),
					'desc' 		=> esc_html__('Redirect user to other page/site if minimum age condition is not met.', 'aat-age-restriction'),
					'std'		=> '',
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				'country_undetected' => array(
					'type' 		=> 'select',
					'std' 		=> 'yes',
					'size' 		=> 'large',
					'force_width'=> '120',
					'title' 	=> esc_html__('Force show if undetected country:', 'aat-age-restriction'),
					'desc' 		=> esc_html__('Show the restriction if the country is not detected.', 'aat-age-restriction'),
					'options'	=> array(
						'no' 			=> esc_html__('No', 'aat-age-restriction'),
						'yes' 			=> esc_html__('Yes', 'aat-age-restriction')
					),
					'cssclass'	=> array('popup')
				),
				'enable_remember_me' => array(
					'type' 		=> 'select',
					'std' 		=> 'yes',
					'size' 		=> 'large',
					'force_width'=> '120',
					'title' 	=> esc_html__('Enable "Remember me" button?', 'aat-age-restriction'),
					'desc' 		=> esc_html__('Disable this if you want your users to be prompted by the verification window each time they acces the page.', 'aat-age-restriction'),
					'options'	=> array(
						'no' 			=> esc_html__('No', 'aat-age-restriction'),
						'yes' 			=> esc_html__('Yes', 'aat-age-restriction')
					),
					'cssclass'	=> array('popup')
				),
				'set_cookie_duration' => array(
					'type' 		=> 'text',
					'size' 		=> 'large',
					'force_width' => 50,
					'title' 	=> esc_html__('Cookies duration (in days):', 'aat-age-restriction'),
					'desc' 		=> esc_html__('How much time should pass until the user will be prompted to  verify his age again (if "Remember me" is activated). Default is 30 days.', 'aat-age-restriction'),
					'std'		=> '30',
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				'selected_countries' 	=> array(
					'type' 		=> 'multiselect_left2right',
					'std' 		=> array(),
					'size' 		=> 'large',
					'rows_visible'	=> 18,
					'force_width'=> '300',
					'title' 	=> esc_html__('Select countries', 'aat-age-restriction'),
					'desc' 		=> esc_html__('Choose on what countries you want the restriction to be displayed.', 'aat-age-restriction'),
					'info'		=> array(
						'left' => esc_html__('All Countries list', 'aat-age-restriction'),
						'right' => esc_html__('Your chosen countries from list', 'aat-age-restriction'),
					),
					'options' 	=> $this->countries_list(),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				'testing_enabled' => array(
					'type' 		=> 'select',
					'std' 		=> 'no',
					'size' 		=> 'large',
					'force_width'=> '120',
					'title' 	=> '<br/>' . esc_html__('Enable testing:', 'aat-age-restriction'),
					'desc' 		=> wp_kses( __('This enables the feature to delete the SESSION/COOKIE every time after successful validation. If this is enabled, after validation just reload the page to see again the restriction.<br/><span style="color:red;">Testing purpose only! Don\'t forget to disable this after you are done testing.</span>', 'aat-age-restriction'), 'post' ),
					'options'	=> array(
						'no' 			=> esc_html__('No', 'aat-age-restriction'),
						'yes' 			=> esc_html__('Yes', 'aat-age-restriction')
					),
					'cssclass'	=> array('popup')
				),
				'enable_for_choose' => array(
					'type' 		=> 'select',
					'std' 		=> 'all',
					'size' 		=> 'large',
					'force_width'=> '220',
					'title' 	=> esc_html__('Enable for:', 'aat-age-restriction'),
					'desc' 		=> '&nbsp;',
					'options'	=> array(
						'all' 			=> esc_html__('Enable for all', 'aat-age-restriction'),
						'me' 			=> esc_html__('Enable only me', 'aat-age-restriction'),
						'iplist' 		=> esc_html__('Enable for ip list', 'aat-age-restriction'),
						//'url' 			=> esc_html__('Enable for url', 'aat-age-restriction'),
						'disable' 		=> esc_html__('Disable', 'aat-age-restriction')
					),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),

				'enable_for_iplist' => array(
					'type' 		=> 'textarea',
					'size' 		=> 'large',
					'title' 	=> esc_html__('Enable for IP List:', 'aat-age-restriction'),
					'std'		=> '',
					'desc' 		=> esc_html__('Enable for IP List (one IP per line).', 'aat-age-restriction'),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				/* various */
				'connect_w_facebook' => array(
					'type' 		=> 'select',
					'std' 		=> 'no',
					'size' 		=> 'large',
					'force_width'=> '120',
					'title' 	=> esc_html__('Connect with Facebook:', 'aat-age-restriction'),
					'desc' 		=> wp_kses( __('Enable the "CONNECT WITH FACEBOOK" button', 'aat-age-restriction') . (!isset($this->config['fb_app_id']) || trim($this->config['fb_app_id']) == '' ? ' <span style="color:red;">' . sprintf( __('For this functionalty to work you must have a Facebook Application ID setup in the <a href="%s">plugin settings section</a>.</span>', 'aat-age-restriction'), (get_admin_url() . 'admin.php?page=age_restriction#!/settings')) : ''), 'post' ),
					'options'	=> array(
						'yes' 	=> esc_html__('YES', 'aat-age-restriction'),
						'no' 	=> esc_html__('NO', 'aat-age-restriction')
					),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				'connect_w_facebook_btnText' => array(
					'type' 		=> 'text',
					'std' 		=> esc_html__('Connect with Facebook', 'aat-age-restriction'),
					'size' 		=> 'large',
					'title' 	=> esc_html__('Facebook button text:', 'aat-age-restriction'),
					'desc' 		=> '&nbsp;',
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				'connect_w_google' => array(
					'type' 		=> 'select',
					'std' 		=> 'no',
					'size' 		=> 'large',
					'force_width'=> '120',
					'title' 	=> esc_html__('Sign in with Google+:', 'aat-age-restriction'),
					'desc' 		=> wp_kses( __('Enable the "SIGN IN WITH GOOGLE" button', 'aat-age-restriction') . (!isset($this->config['google_client_id']) || trim($this->config['google_client_id']) == '' ? ' <span style="color:red;">'. sprintf( __('For this functionalty to work you must have a Google Client ID setup in the <a href="%s">plugin settings section</a>.</span>', 'aat-age-restriction'), (get_admin_url() . 'admin.php?page=age_restriction#!/settings')) : ''), 'post' ),
					'options'	=> array(
						'yes' 	=> esc_html__('YES', 'aat-age-restriction'),
						'no' 	=> esc_html__('NO', 'aat-age-restriction')
					),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				'connect_w_google_btnText' => array(
					'type' 		=> 'text',
					'std' 		=> esc_html__('Connect with Google+', 'aat-age-restriction'),
					'size' 		=> 'large',
					'title' 	=> esc_html__('Google+ button text:', 'aat-age-restriction'),
					'desc' 		=> '&nbsp;',
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				
				/* Meta Tags */
				'page_title' => array(
					'type' 		=> 'text',
					'std' 		=> esc_html__('Verify your age', 'aat-age-restriction'),
					'size' 		=> 'large',
					'title' 	=> esc_html__('SEO Page Title:', 'aat-age-restriction'),
					'desc' 		=> esc_html__('( optional: if the crawler robots cannot bypass age restriction and read your site meta tags )', 'aat-age-restriction'),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				'meta_description' => array(
					'type' 		=> 'text',
					'std' 		=> '',
					'size' 		=> 'large',
					'title' 	=> esc_html__('SEO Description:', 'aat-age-restriction'),
					'desc' 		=> esc_html__('( optional: if the crawler robots cannot bypass age restriction and read your site meta tags )', 'aat-age-restriction'),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
				'meta_keywords' => array(
					'type' 		=> 'text',
					'std' 		=> '',
					'size' 		=> 'large',
					'title' 	=> esc_html__('SEO Keywords:', 'aat-age-restriction'),
					'desc' 		=> esc_html__('( optional: if the crawler robots cannot bypass age restriction and read your site meta tags )', 'aat-age-restriction'),
					'cssclass'	=> array('popup', 'widget', 'slide', 'content_top', 'content_bottom', 'bar')
				),
			);
			 
			foreach ($this->elements as $field_key => $field_val) {
				$cssclass = $field_val['cssclass'];
				$cssclass = implode(',', array_map(array($this, "do_prefix"), $cssclass));
				$this->elements["$field_key"]['cssclass'] = $cssclass;
			}
			
			// tabs
			$this->tabs = array(
				'__tab1'	=> array(
					esc_html__('Design', 'aat-age-restriction'),
					'theme, logo, background_image, background_color, box_background_color, box_background_opacity, text_color, text_hover_color, country_selection, date_format, gdpr_consent, gdpr_consent_text, gdpr_error_consent_text, gdpr_agree_checkbox_label'
				),
				'__tab2'	=> array(
					esc_html__('Content', 'aat-age-restriction'),
					'typography, restriction_title, title_text_color, text_before, text_color_before, text_after, text_color_after, enter_btn_title, enter_btn_bg_color, enter_btn_text_color, minimum_age_error_message'
				),
				'__tab3'	=> array(
					esc_html__('Custom CSS', 'aat-age-restriction'),
					'custom_css'
				),
				'__tab4'	=> array(
					esc_html__('Limitations', 'aat-age-restriction'),
					'minimum_age, redirect_under_age, country_undetected, enable_remember_me, set_cookie_duration, selected_countries, testing_enabled, enable_for_choose, enable_for_iplist, enable_for_url', //test_enabled, test_ip'
				),
				'__tab6'	=> array(
					esc_html__('Social Connect', 'aat-age-restriction'),
					'connect_w_facebook, connect_w_google, connect_w_facebook_btnText, connect_w_google_btnText'
				),
				'__tab7'	=> array(
					esc_html__('SEO', 'aat-age-restriction'),
					'page_title, meta_description, meta_keywords'
				)
			);
		}

		public function get_elements() {
			$this->setup();
			return $this->elements;
		}
		
		public function get_tabs() {
			$this->setup();
			return $this->tabs;
		}
		
		
		/**
		 * Extra
		 */
		public function set_banner_postid($postid) {
			$this->postid = (int) $postid;
		}
		
		
		/**
		 * Banner type specific
		 */
		private function do_prefix($arr) {
			return 'btype-' . $arr;
		}

		private function countries_list() {
			require($this->module_folder_path . 'lists.inc.php');
			return $age_restriction_countries_list;
		}

		private function get_range_opacity() {
			$arr = array();
			for ($i=0; $i<=1; $i += 0.1) {
				$arr["$i"] = $i;
			}
			return $arr;
		}
	}
}

//new age_restrictionBannersType();